from sanic.views import HTTPMethodView
from sanic.response import text
from sanic.response import json
from pprint import pprint


class BaseView(HTTPMethodView):
  def __init__(self, queue, rpc_client, params):
      self.queue = queue
      self.rpc_client = rpc_client
      self.params = params

  def request_to_message(self, request, optype):
      message = dict()
      message["optype"] = optype
      if self.params > 0:
          try:
              message["json_content"] = request.json
          except Exception:
              pass
      if self.params > 1:
          message["url_content"] = request.args
      if self.params > 2:
          message["form_content"] = request.form
      return message

  async def get(self, request):
      response = self.rpc_client.call(self.request_to_message(request, "GET"), self.queue)
      return json(eval(response))

  async def post(self, request):

      response = self.rpc_client.call(self.request_to_message(request, "POST"), self.queue)
      return json(eval(response))

  async def put(self, request):
      response = self.rpc_client.call(self.request_to_message(request, "PUT"), self.queue)
      return json(eval(response))

  async def patch(self, request):
      response = self.rpc_client.call(self.request_to_message(request, "PATCH"), self.queue)
      return json(eval(response))

  async def delete(self, request):
      response = self.rpc_client.call(self.request_to_message(request, "DELETE"), self.queue)
      return json(eval(response))



# -*- coding: UTF-8 -*-

import pika
import types
import uuid
import time
from pprint import pprint
import json
import configparser
import uuid

from multiprocessing import Process

#INI_PATH = "/home/luze/Enigma/enigma/config/enigma-server.ini"

HOST = "localhost"
EXCHANGE = "allegro::%s" % str(uuid.uuid1()).replace("-","")[:10]
PREFETCH = 2


class RpcClient(object):
    def __init__(self, timeout):
        self.timeout = timeout
        self.connection = pika.BlockingConnection(pika.ConnectionParameters(
            host=HOST, heartbeat_interval=0))
        self.channel = self.connection.channel()
        result = self.channel.queue_declare(exclusive=True)
        self.callback_queue = result.method.queue
        self.channel.exchange_declare(exchange=EXCHANGE,type='direct')
        self.channel.basic_consume(self.on_response, no_ack=True,
                                   queue=self.callback_queue)

    def on_response(self, ch, method, props, body):
        if self.corr_id == props.correlation_id:
            self.response = body


    def call(self, message, queue):
        self.response = None
        self.corr_id = str(uuid.uuid4())
        self.channel.basic_publish(exchange=EXCHANGE,
                                   routing_key=queue,
                                   properties=pika.BasicProperties(
                                       reply_to=self.callback_queue,
                                       correlation_id=self.corr_id,
                                   ),
                                   body=str(message))
        start = time.time()
        while self.response is None:
            self.connection.process_data_events()
            if time.time()-start > self.timeout: #if timeout raise error
                self.response = {'info':'time out', 'state':0}
        return self.response



class RpcServer(Process):
    def __init__(self):
        super(RpcServer, self).__init__()


    def _init_rpc(self, amqp, queue):
        connection = pika.BlockingConnection(pika.ConnectionParameters(host=amqp))
        self.channel = connection.channel()
        self.channel.exchange_declare(exchange=EXCHANGE, type='direct')

        self.channel.basic_qos(prefetch_count=PREFETCH)
        result =  self.channel.queue_declare(queue=queue, durable=True)
        self.channel.queue_bind(exchange=EXCHANGE,
                                queue=result.method.queue,
                                routing_key=queue)
        self.channel.basic_consume(self._on_request, queue=queue, no_ack=False)


    def _on_request(self, ch, method, props, body):
        message = eval(body)

        response = self.handler_request(message)
        ch.basic_publish(exchange='',
                         routing_key=props.reply_to,
                         properties=pika.BasicProperties(correlation_id= \
                                                         props.correlation_id),
                         body=str(response))
        ch.basic_ack(delivery_tag=method.delivery_tag)

    def handler_request(self, message):

        return

    def _request_handler(self, conn, type, message):
        if type == "GET" :
            print("GET")
            return conn.get(message)
        elif type == "POST":
            print("POST")
            return conn.post(message)
        elif type == "UPDATE":
            return conn.update(message)
        elif type == "DELETE":
            return conn.delete(message)
        elif type == "PATCH":
            return conn.patch(message)
        elif type == "PUT":
            return conn.put(message)
        else:
            reponse = "Error: the optype is not valid."
            return reponse


    def run(self):
        self.channel.start_consuming()
                                                        



from sanic import Sanic
from sanic.response import json
from sanic.config import Config
from sanic.response import text

import os
import configparser
import signal

from .controller import BaseView
from .rpc import RpcClient
import logging

from asyncio import get_event_loop

from .service import BaseService

#Config.REQUEST_TIMEOUT = 2
class Allegro(object):
    def __init__(self, name):
        self.log = logging.getLogger('allegro')

        formatter = logging.Formatter(
                    "%(asctime)s: %(levelname)s: %(message)s")
        handler = logging.StreamHandler()
        handler.setFormatter(formatter)
        self.log.addHandler(handler)
        self.log.setLevel(logging.INFO)

        self.app = Sanic(name)
    
    def initialize(self, config_path):
         
        try:
            self.cf = configparser.ConfigParser()
            self.cf.read(config_path)
         
            self.host = self.cf.get("default", "bind_host")
            self.port = self.cf.getint("default", "bind_port")
            self.wokers = self.cf.getint("default", "api_worker")
            self.consumer_path = self.cf.get("default", "consumer_path")
            self.pid_path = self.cf.get("default", "pid_path")
            self.timeout = self.cf.getint("default", "timeout")
            self.app.config.REQUEST_TIMEOUT=self.timeout

        except Exception as e:
            self.log.exception("Error ocurred when trying to the config file Info: %s" % str(e))
            raise

    def init_route(self):
        services = self.cf.get("service", "keys").split(',')
        for service in services:
            rpc_client = RpcClient(self.timeout)
            queue = self.cf.get(service, "queue") 
            uri = self.cf.get(service, "uri")
            params =  self.cf.getint(service, "params")
            self.app.add_route(BaseView.as_view(queue, rpc_client, params), uri)

    def start(self):
        try:
            self.init_route()
            with open(self.pid_path, "a") as f:
                f.write(str(os.getpid())+"\n")

            #self.app.run(host=self.host, port=self.port, workers=self.wokers)
            self.log.info("Starting Consumer service...")
            services = self.cf.get("service", "keys").split(',')
            for service in services:
                processList = []
                workers = self.cf.getint(service, "workers")
                consumer = self.cf.get(service, "consumer")
                queue = self.cf.get(service, "queue")
                module = self.cf.get(service, "module")
                for i in range(workers) :
                    try:
                        p = BaseService(queue, consumer, module, self.consumer_path)
                    except Exception as e:
                        self.log.exception("Failed to import %s module from %s" % (self.consumer))
                        raise
                    processList.append(p)
                    p.start()
                for p in processList:
                    with open(self.pid_path, "a") as f:
                        f.write(str(p.pid)+"\n")
            self.log.info("Stiiiarting API service...")
            self.app.add_task(self.save_pid())
            self.app.run(host=self.host, port=self.port, workers=self.wokers)


        except Exception as e:
            self.log.exception("Error ocurred when trying to the config file Info: %s" % str(e))
            raise

    async def save_pid(self):
        with open(self.pid_path, "a") as f:
            f.write(str(os.getpid())+"\n")

    def stop(self):
        try:
            self.log.info("Staring to termina the processes...")
            with open(self.pid_path, "r") as f:
                processes = f.readlines()
                for p in processes:
                    try:
                        a = os.kill(int(p[:-1]), signal.SIGKILL)
                    except Exception as e:
                        self.log.error(e)
            open(self.pid_path, "w+")
            self.log.info("All the processes are terminated.")
        except Exception as e:
            log.exception("Error ocurred when trying to kill the processes Info: %s" % str(e)) 
            raise




import imp
from . import rpc

class BaseService(rpc.RpcServer):
    def __init__(self, queue, consumer, module, path):
        super(BaseService, self).__init__()

        self.queue = queue
        self._init_rpc("localhost", self.queue)
        try:
            file, pathname, desc = imp.find_module(module,[path])
            self.moduleobj = imp.load_module(consumer, file, pathname, desc)
            self.consumer = "self.moduleobj.%s" % consumer
        except Exception as e:
            raise

    def handler_request(self, message):
        """handle a specific request from amqp::templates
        """
        body = dict()
        optype = message.pop("optype")
        consumer = eval(self.consumer)()
        for content in message.keys():
            if message[content] != {} and message[content] != None:
                body[content] = message[content]
        return self._request_handler(consumer, optype, body)




"""
Sanic
"""
import codecs
import os
import re
from distutils.errors import DistutilsPlatformError
from distutils.util import strtobool

from setuptools import setup

with codecs.open(os.path.join(os.path.abspath(os.path.dirname(
        __file__)), 'allegro', '__init__.py'), 'r', 'latin1') as fp:
    try:
        version = re.findall(r"^__version__ = '([^']+)'\r?$",
                             fp.read(), re.M)[0]
    except IndexError:
        raise RuntimeError('Unable to determine version.')

setup_kwargs = {
    'name': 'Allegro',
    'version': version,
    'url': 'https://github.com/Alecyrus/Allegro',
    'license': 'MIT',
    'author': 'Alecyrus',
    'author_email': 'alecyrus@163.com',
    'description': (
        'A python backend integration framework'),
    'packages': ['allegro'],
    'platforms': 'any',
    'classifiers': [
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5'
    ],
}

requirements = [
    'pika>=0.10.0',
    'sanic>=0.5.1',
]
try:
    setup_kwargs['install_requires'] = requirements
    setup(**setup_kwargs)
except DistutilsPlatformError as exception:
    setup_kwargs['install_requires'] = requirements
    setup(**setup_kwargs)
